@@ -6,9 +6,25 @@ from itertools import cycle
 _builtin_open = open
 
 class ShapeList(list):
-    def __init__(self, *ka, **kw):
-        self._comment_list = kw.pop("comment_list", None)
-        list.__init__(self, *ka, **kw)
+    """ A list of shape objects """
+    def __init__(self, shape_list, comment_list=None):
+        """
+
+        Parameters
+        ----------
+        shape_list : a list of 'pyregion.parse_helper.Shape' objects
+        comment_list : list, None
+         list of comment strings for each argument
+
+
+        """
+        if comment_list is not None:
+            if len(comment_list) != len(shape_list):
+                err = "Ambiguous number of comments {} for number of shapes {}"
+                raise ValueError(err.format(len(comment_list),
+                                            len(shape_list)))
+        self._comment_list = comment_list
+        list.__init__(self, shape_list)
 
     def __getitem__(self, key):
         if isinstance(key, slice):
@@ -25,7 +41,7 @@ class ShapeList(list):
         else:
             return True
 
-    def as_imagecoord(self, header):
+    def as_imagecoord(self, header, rot_wrt_axis=1):
         """
         Return a new ShapeList where the coordinate of the each shape
         is converted to the image coordinate using the given header
@@ -37,11 +53,10 @@ class ShapeList(list):
             comment_list = cycle([None])
 
         r = RegionParser.sky_to_image(zip(self, comment_list),
-                                      header)
+                                      header, rot_wrt_axis=rot_wrt_axis)
         shape_list, comment_list = zip(*list(r))
         return ShapeList(shape_list, comment_list=comment_list)
 
-
     def get_mpl_patches_texts(self, properties_func=None,
                               text_offset=5.0,
                               origin=1):
@@ -60,7 +75,7 @@ class ShapeList(list):
 
         return patches, txts
 
-    def get_filter(self, header=None, origin=1):
+    def get_filter(self, header=None, origin=1, rot_wrt_axis=1):
         """
         Often, the regions files implicitly assume the lower-left
         corner of the image as a coordinate (1,1). However, the python
@@ -77,14 +92,14 @@ class ShapeList(list):
                 raise RuntimeError("the region has non-image coordinate. header is required.")
             reg_in_imagecoord = self
         else:
-            reg_in_imagecoord = self.as_imagecoord(header)
+            reg_in_imagecoord = self.as_imagecoord(header, rot_wrt_axis=rot_wrt_axis)
 
         region_filter = as_region_filter(reg_in_imagecoord, origin=1)
 
         return region_filter
 
 
-    def get_mask(self, hdu=None, header=None, shape=None):
+    def get_mask(self, hdu=None, header=None, shape=None, rot_wrt_axis=1):
         """
         creates a 2-d mask.
 
@@ -98,34 +113,66 @@ class ShapeList(list):
         if hdu and shape is None:
             shape = hdu.data.shape
 
-        region_filter = self.get_filter(header=header)
+        region_filter = self.get_filter(header=header, rot_wrt_axis=rot_wrt_axis)
         mask = region_filter.mask(shape)
 
         return mask
 
-    def write(self,outfile):
+
+    def write(self, outfile):
         """ Writes the current shape list out as a region file """
+        if len(self) < 1:
+            print("WARNING: The region list is empty. The region file "\
+                  "'{:s}' will be empty.".format(outfile))
+            try:
+                outf = open(outfile,'w')
+                outf.close()
+                return
+            except IOError as e:
+                cmsg = "Unable to create region file '{:s}'.".format(outfile)
+                if e.args:
+                    e.args = (e.args[0] + '\n' + cmsg,) + e.args[1:]
+                else:
+                    e.args=(cmsg,)
+                raise e
 
-        # check for consistent coordinate system
-        if len(set([shape.coord_format for shape in self])) > 1:
-            raise ValueError("Inconsistent coordinate formats")
+        prev_cs = self[0].coord_format
 
-        outf = _builtin_open(outfile,'w')
+        outf = None
+        try:
+            outf = _builtin_open(outfile,'w')
 
-        attr0 = self[0].attr[1]
-        defaultline = " ".join(["%s=%s" % (a,attr0[a]) for a in attr0 if a!='text'])
+            attr0 = self[0].attr[1]
+            defaultline = " ".join( [ "{:s}={:s}".format(a,attr0[a]) \
+                                      for a in attr0 if a != 'text' ] )
 
-        # first line is globals
-        print >>outf,"global",defaultline
-        # second line must be a coordinate format
-        print >>outf,self[0].coord_format
+            # first line is globals
+            print >>outf, "global", defaultline
+            # second line must be a coordinate format
+            print >>outf, prev_cs
 
-        for shape in self:
-            text_coordlist = ["%f" % f for f in shape.coord_list]
-            print >>outf, \
-                shape.name + "(" + ",".join(text_coordlist) + ") # " + shape.comment
+            for shape in self:
+                shape_attr = '' if prev_cs == shape.coord_format \
+                    else shape.coord_format+"; "
+                shape_excl = '-' if shape.exclude else ''
+                text_coordlist = [ "{:f}".format(f) for f in shape.coord_list ]
+                shape_coords = "(" + ",".join(text_coordlist) + ")"
+                shape_comment = " # " + shape.comment if shape.comment else ''
 
-        outf.close()
+                shape_str = shape_attr + shape_excl + shape.name + \
+                            shape_coords + shape_comment
+
+                print >>outf, shape_str
+
+        except IOError as e:
+            cmsg = "Unable to create region file \'{:s}\'.".format(outfile)
+            if e.args:
+                e.args = (e.args[0] + '\n' + cmsg,) + e.args[1:]
+            else:
+                e.args=(cmsg,)
+            raise e
+        finally:
+            if outf: outf.close()
 
 
 def parse(region_string):
@@ -160,33 +207,27 @@ def read_region(s):
     return rp.filter_shape(sss2)
 
 
-def read_region_as_imagecoord(s, header):
+def read_region_as_imagecoord(s, header, rot_wrt_axis=1):
     rp = RegionParser()
     ss = rp.parse(s)
     sss1 = rp.convert_attr(ss)
     sss2 = _check_wcs(sss1)
-    sss3 = rp.sky_to_image(sss2, header)
+    sss3 = rp.sky_to_image(sss2, header, rot_wrt_axis=rot_wrt_axis)
 
     return rp.filter_shape(sss3)
 
 
-
-def get_mask(region, hdu):
+def get_mask(region, hdu, origin=1):
     """
     f = pyfits.read("test.fits")
     reg = read_region_as_imagecoord(s, f[0].header)
     mask = get_mask(reg, f[0])
     """
-
     from pyregion.region_to_filter import as_region_filter
 
     data = hdu.data
-    header = hdu.header
-
-    region_filter = as_region_filter(region, header=header)
-
+    region_filter = as_region_filter(region, origin=origin)
     mask = region_filter.mask(data)
-
     return mask
 
 
